## Context

{{ context | default('') }}

## Choice

{{ choice | default('') }}

## Rationale

{{ rationale | default('') }}

## Alternatives

{{ alternatives | default('') }}

## Consequences

{{ consequences | default('') }}
