---
generated: true
vault: "{{ vault_name }}"
created: {{ created }}
---

# Polaris — {{ vault_name }}

Your polaris is your north star. Update it as your priorities evolve.

## Mission

<!-- What are you ultimately trying to accomplish? Write one to three sentences. -->

## Current Priorities

1. <!-- Priority one -->
2. <!-- Priority two -->
3. <!-- Priority three -->

## Decision Principles

- <!-- Principle one: what criteria guide your decisions? -->
- <!-- Principle two: what do you consistently optimize for? -->
- <!-- Principle three: what do you consistently avoid? -->
