## Summary
{% if summary is defined and summary %}
{{ summary }}
{% elif body is defined and body %}
{{ body }}
{% endif %}

## Key Points
{% if key_points is defined and key_points %}
{% for item in key_points %}
- {{ item }}
{% endfor %}
{% endif %}

## Excerpts
{% if excerpts is defined and excerpts %}
{% for excerpt in excerpts %}
> {{ excerpt | replace('\n', '\n> ') }}

{% endfor %}
{% endif %}

## Notes
{% if notes is defined and notes %}
{{ notes }}
{% endif %}

## Provenance
{% if url is defined and url %}
- URL: {{ url }}
{% endif %}
{% if canonical_url is defined and canonical_url and canonical_url != url %}
- Canonical URL: {{ canonical_url }}
{% endif %}
{% if source_provider is defined and source_provider %}
- Provider: {{ source_provider }}
{% endif %}
{% if source_type is defined and source_type %}
- Source Type: {{ source_type }}
{% endif %}
{% if source_kind is defined and source_kind %}
- Source Kind: {{ source_kind }}
{% endif %}
{% if modalities is defined and modalities %}
- Modalities: {{ modalities | join(', ') }}
{% endif %}
{% if capture_agent is defined and capture_agent %}
- Capture Agent: {{ capture_agent }}
{% endif %}
{% if capture_method is defined and capture_method %}
- Capture Method: {{ capture_method }}
{% endif %}
{% if retrieved_at is defined and retrieved_at %}
- Retrieved At: {{ retrieved_at }}
{% endif %}
{% if content_hash is defined and content_hash %}
- Content Hash: {{ content_hash }}
{% endif %}
{% if language is defined and language %}
- Language: {{ language }}
{% endif %}
{% if provenance is defined and provenance %}
{% for item in provenance %}
- {{ item }}
{% endfor %}
{% endif %}
{% if citations is defined and citations %}
{% for citation in citations %}
- Citation: {{ citation }}
{% endfor %}
{% endif %}
{% if artifacts is defined and artifacts %}
{% for artifact in artifacts %}
- Artifact{% if artifact.label is defined and artifact.label %} {{ artifact.label }}{% endif %}{% if artifact.kind is defined and artifact.kind %} ({{ artifact.kind }}){% endif %}{% if artifact.uri is defined and artifact.uri %}: {{ artifact.uri }}{% endif %}
{% endfor %}
{% endif %}
{% if source_bundle_path is defined and source_bundle_path %}
- Source Bundle: {{ source_bundle_path }}
{% endif %}
